﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.Design;
using System.Drawing.Design;
using System.IO;
using System.Linq;
using System.Net;
using System.Security.Cryptography;
using System.Text;
using System.Threading.Tasks;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class BaseRequest
    {
        protected string _methodName = string.Empty; // e.g. "create"

        public string GetURL(string MethodName, bool IsForEncrypt = false)
        {
            string url = IsForEncrypt ? MethodName : MethodName + "?";
            foreach (DictionaryEntry de in _parameters)
            {
                string value = (string)de.Value;
                value = value.Replace(" ", "+");
                url += de.Key + "=" + value + "&";
            }

            url = url.TrimEnd('&');
            string checksum = string.Empty;
            if (!IsForEncrypt)
            {
                string UrlForSHA = GetURL(MethodName, true);
                UrlForSHA = UrlForSHA + this._securitySalt;
                checksum = General.GetSHA1(UrlForSHA);
                url = url + "&checksum=" + checksum;
                url = this._serverURL + url;
            }

            _requestURL = url;
            return url;
        }

        public string GetURL(bool IsForEncrypt = false)
        {
            return this.GetURL(this._methodName, IsForEncrypt);
        }

        protected Hashtable _parameters = new Hashtable(StringComparer.InvariantCultureIgnoreCase);
        public Hashtable Parameters
        {
            get { return _parameters; }
        }

        public string GetParamByName(string Name)
        {
            Name = Name.Trim();
            if (!String.IsNullOrEmpty(Name) && _parameters.ContainsKey(Name))
            {
                return (string)_parameters[Name];
            }
            return null;
        }

        public void AddParameters(string key, string value)
        {
            if (!String.IsNullOrEmpty(key) && !String.IsNullOrEmpty(value))
            {
                key = key.Trim();
                if (_parameters.ContainsKey(key))
                    _parameters[key] = value;
                else
                    _parameters.Add(key, value);
                this.GetURL();
            }
        }

        public static bool StrToBool(string value)
        {
            if (value != null)
                return value.Trim().ToLower() == "true" ? true : false;
            else
                return false;
        }


        private string _requestURL;
        [Category("Result")]
        [DisplayName("Request URL")]
        [Description("URL to sent to BigBlueButton server.")]
        [Editor(typeof(MultilineStringEditor), typeof(UITypeEditor))]
        public string RequestURL
        {
            get
            {
                return _requestURL;
            }
        }

        private string _securitySalt;
        [DisplayName("Security salt (shared secret)")]
        [Description("All API calls include a checksum that signs the parameters with a shared secret (called salt by BigBlueButton).")]
        public string SecuritySalt
        {
            get { return _securitySalt; }
            set
            {
                _securitySalt = value;
                this.GetURL();
            }
        }

        private string _serverURL;
        [DisplayName("Server URL")]
        //[Description("Server URL.")]
        public string ServerURL
        {
            get { return _serverURL; }
            set
            {
                _serverURL = value.TrimEnd('/') + "/";
                if (!_serverURL.EndsWith("/api/"))
                    _serverURL += "api/";
                this.GetURL();
            }
        }

        protected BaseResponse _response;
        public virtual object Execute()
        {
            try
            {
                HttpWebRequest request = (HttpWebRequest)WebRequest.Create(this.RequestURL);
                HttpWebResponse response = (HttpWebResponse)request.GetResponse();
                StreamReader sr = new StreamReader(response.GetResponseStream());
                if (sr != null)
                {
                    string responseXML = sr.ReadToEnd();
                    _response.Response = responseXML;
                    return _response;
                }
                return null;
            }
            catch (Exception err)
            {
                return null;
            }
        }
    }
}
