﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class EndMeetingResponse : BaseResponse
    {
    }

    public class EndMeetingRequest : BaseRequest
    {
        public EndMeetingRequest()
            : base()
        {
            this._methodName = "end";
            this._response = new EndMeetingResponse();
        }

        public new EndMeetingResponse Execute()
        {
            return (EndMeetingResponse)base.Execute();
        }

        /// <summary>
        /// Required	A meeting ID that can be used to identify this meeting by the 3rd-party application.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("A meeting ID that can be used to identify this meeting by the 3rd-party application.")]
        public string MeetingID
        {
            get { return GetParamByName("meetingID"); }
            set { AddParameters("meetingID", value); }
        }

        /// <summary>
        /// Required	The moderator password for this meeting. You can not end a meeting using the attendee password.
        /// </summary>
        [DisplayName("Password (required)")]
        [Description("The moderator password for this meeting. You can not end a meeting using the attendee password.")]
        public string Password
        {
            get { return GetParamByName("password"); }
            set { AddParameters("password", value); }
        }
    }

    public class EndMeeting
    {
        private EndMeetingRequest _request;
        [CategoryAttribute("End Meeting Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("End Meeting request")]
        public EndMeetingRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private EndMeetingResponse _response;
        [CategoryAttribute("End Meeting Response")]
        [DisplayName("End Meeting response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public EndMeetingResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
