﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class GetMeetingInfoRequest : BaseRequest
    {
        public GetMeetingInfoRequest()
            : base()
        {
            this._methodName = "getMeetingInfo";
            this._response = new GetMeetingInfoResponse();
        }

        public new GetMeetingInfoResponse Execute()
        {
            return (GetMeetingInfoResponse)base.Execute();
        }

        /// <summary>
        /// Required	A meeting ID that can be used to identify this meeting by the 3rd-party application.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("A meeting ID that can be used to identify this meeting by the 3rd-party application.")]
        public string MeetingID
        {
            get { return GetParamByName("meetingID"); }
            set { AddParameters("meetingID", value); }
        }

        /// <summary>
        /// Required	The moderator password for this meeting. You can not end a meeting using the attendee password.
        /// </summary>
        [DisplayName("Password (required)")]
        [Description("The moderator password for this meeting. You can not end a meeting using the attendee password.")]
        public string Password
        {
            get { return GetParamByName("password"); }
            set { AddParameters("password", value); }
        }
    }

    public class GetMeetingInfoResponse : BaseResponse
    {
        protected override void ProcessChildNodes(XmlNode node, Hashtable hashtable)
        {
            switch (node.Name.ToLower())
            {
                case "attendees":
                    _attendees.Clear();
                    Attendee _attendee = new Attendee();
                    foreach (XmlNode attendees in node.ChildNodes)
                    {
                        foreach (XmlNode attendee in attendees.ChildNodes)
                        {
                            _attendee.AddHashTable(attendee.Name, attendee.InnerText);
                        }
                        _attendee.Response = node.InnerXml;
                        _attendees.Add(_attendee);
                    }
                    break;
                case "metadata":
                    foreach (XmlNode metadata in node.ChildNodes)
                    {
                        _metadata.AddHashTable(metadata.Name, metadata.InnerText);
                    }
                    _metadata.Response = node.OuterXml;
                    break;
            }
            //base.ProcessChildNodes(node, hashtable);
        }

        private Attendees _attendees = new Attendees();
        public Attendees Attendees
        {
            get { return _attendees; }
        }

        //private 

        public DateTime CreateTime
        {
            get
            {
                string createTime = GetValueByName("createTime");
                return General.MillisecondsToDateTime(createTime);
            }
        }

        public string VoiceBridge
        {
            get { return GetValueByName("voiceBridge"); }
        }

        public bool IsRunning
        {
            get
            {
                string running = GetValueByName("running").ToLower();
                return running == "true";
            }
        }

        public bool IsRecording
        {
            get
            {
                string recording = GetValueByName("recording").ToLower();
                return recording == "true";
            }
        }

        public bool IsForciblyEnded
        {
            get
            {
                string HasBeenForciblyEnded = GetValueByName("hasBeenForciblyEnded").ToLower();
                return HasBeenForciblyEnded == "true";
            }
        }

        public DateTime StartTime
        {
            get
            {
                string startTime = GetValueByName("startTime");
                return General.MillisecondsToDateTime(startTime);
            }
        }

        public DateTime EndTime
        {
            get
            {
                string endTime = GetValueByName("endTime");
                return General.MillisecondsToDateTime(endTime);
            }
        }

        public int ParticipantCount
        {
            get
            {
                int count = 0;
                string participans = GetValueByName("participantCount");
                if (!string.IsNullOrEmpty(participans))
                    int.TryParse(participans, out count);
                return count;
            }
        }

        public int MaxUsers
        {
            get
            {
                int count = 0;
                string participans = GetValueByName("maxUsers");
                if (!string.IsNullOrEmpty(participans))
                    int.TryParse(participans, out count);
                return count;
            }
        }

        public int ModeratorCount
        {
            get
            {
                int count = 0;
                string participans = GetValueByName("moderatorCount");
                if (!string.IsNullOrEmpty(participans))
                    int.TryParse(participans, out count);
                return count;
            }
        }

        private Metadata _metadata = new Metadata();
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public Metadata Metadata
        {
            get { return _metadata; }
        }


        //private void ParseAttendees()
        //{
        //    string attendees = GetValueByName("attendees");
        //    if (!string.IsNullOrEmpty(attendees))
        //    {
        //        this.FillHashTable(attendees, "attendee");//, ref _hashtable);
        //    }
        //}

        //private Hashtable _hashtable = new Hashtable(StringComparer.InvariantCultureIgnoreCase);
        //public Hashtable attendees
        //{
        //    get
        //    {
        //        string attendees = GetValueByName("attendees");
        //        if (!string.IsNullOrEmpty(attendees))
        //        {
        //            this.FillHashTable(attendees, "attendees", ref _hashtable);
        //        }
        //        return _hashtable;
        //    }
        //}
    }

    public class Attendees : List<Attendee>
    {

    }

    public class Attendee : BaseResponse
    {
        /// <summary>
        /// Required	The full name that is to be used to identify this user to other conference attendees.
        /// </summary>
        [DisplayName("Full Name (required)")]
        [Description("The full name that is to be used to identify this user to other conference attendees.")]
        public string FullName
        {
            get 
            { 
                string encoded = GetValueByName("fullName");
                return WebUtility.UrlDecode(encoded);
            }
            set { this.AddHashTable("fullName", value); }
        }

        public string UserID
        {
            get { return GetValueByName("userID"); }
            set { this.AddHashTable("userID", value); }
        }

        public string Role
        {
            get { return GetValueByName("role"); }
            set { this.AddHashTable("role", value); }
        }

        public string Customdata
        {
            get { return GetValueByName("customdata"); }
            set { this.AddHashTable("customdata", value); }
        }
    }

    public class MetadataRecords : BaseResponse
    {
        public string Originurl
        {
            get { return GetValueByName("originurl"); }
            //set { this.AddHashTable("originurl", value); }
        }

        public string Originversion
        {
            get { return GetValueByName("originversion"); }
            //set { this.AddHashTable("originversion", value); }
        }

        public string Originservercommonname
        {
            get { return GetValueByName("originservercommonname"); }
            //set { this.AddHashTable("originservercommonname", value); }
        }

        public string Originservername
        {
            get { return GetValueByName("originservername"); }
            //set { this.AddHashTable("originservername", value); }
        }

        public string Origin
        {
            get { return GetValueByName("origin"); }
            //set { this.AddHashTable("origin", value); }
        }

        public string Origintag
        {
            get { return GetValueByName("origintag"); }
            //set { this.AddHashTable("origintag", value); }
        }
    }

    public class GetMeetingInfo
    {
        private GetMeetingInfoRequest _request;
        [CategoryAttribute("Get Meeting Info Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Get Meeting Info request")]
        public GetMeetingInfoRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private GetMeetingInfoResponse _response;
        [CategoryAttribute("Get Meeting Info Response")]
        [DisplayName("Get Meeting Info response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public GetMeetingInfoResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
