﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class GetMeetingsRequest : BaseRequest
    {
        public GetMeetingsRequest()
        {
            this._methodName = "getMeetings";
            this._response = new GetMeetingsResponse();
        }

        public new GetMeetingsResponse Execute()
        {
            return (GetMeetingsResponse)base.Execute();
        }
    }

    public class GetMeetingsResponse : BaseResponse
    {
        private Meetings _meetings = new Meetings();
        public Meetings Meetings
        {
            get { return _meetings; }
        }

        protected override void ProcessChildNodes(XmlNode node, Hashtable hashtable)
        {
            if (node.Name.ToLower() == "meetings")
            {
                _meetings.Clear();
                Meeting _meeting = new Meeting();
                foreach (XmlNode meetings in node.ChildNodes)
                {
                    foreach (XmlNode meeting in meetings.ChildNodes)
                    {
                        _meeting.AddHashTable(meeting.Name, meeting.InnerText);
                    }
                    _meeting.Response = node.InnerXml;
                    _meetings.Add(_meeting);
                }
            }
            //base.ProcessChildNodes(node, hashtable);
        }
    }

    public class Meetings : List<Meeting>
    { }

    public class Meeting : BaseResponse
    {
        public string MeetingID
        {
            get { return GetValueByName("meetingID"); }
            set { this.AddHashTable("meetingID", value); }
        }

        public string MeetingName
        {
            get { return GetValueByName("meetingName"); }
            set { this.AddHashTable("meetingName", value); }
        }

        public DateTime CreateTime
        {
            get 
            { 
                string createTime = GetValueByName("createTime");
                return General.MillisecondsToDateTime(createTime);
            }
            set 
            {
                string mills = General.DateTimeToMilliseconds(value);
                this.AddHashTable("createTime", mills); 
            }
        }

        public string AttendeePW
        {
            get { return GetValueByName("attendeePW"); }
            set { this.AddHashTable("attendeePW", value); }
        }

        public string ModeratorPW
        {
            get { return GetValueByName("moderatorPW"); }
            set { this.AddHashTable("moderatorPW", value); }
        }

        public bool Running
        {
            get 
            {
                string running = GetValueByName("running");
                return running.ToLower() == "true";
            }
            set { this.AddHashTable("running", value.ToString()); }
        }

        public bool HasBeenForciblyEnded
        {
            get
            {
                string running = GetValueByName("hasBeenForciblyEnded");
                return running.ToLower() == "true";
            }
            set { this.AddHashTable("hasBeenForciblyEnded", value.ToString()); }
        }
        }

    public class GetMeetings
    {
        private GetMeetingsRequest _request;
        [CategoryAttribute("Get Meeting Info Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Get Meeting Info request")]
        public GetMeetingsRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private GetMeetingsResponse _response;
        [CategoryAttribute("Get Meeting Info Response")]
        [DisplayName("Get Meeting Info response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public GetMeetingsResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
