﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class GetRecordingsRequest : BaseRequest
    {
        public GetRecordingsRequest()
        {
            this._methodName = "getRecordings";
            this._response = new GetRecordingsResponse();
        }

        public new GetRecordingsResponse Execute()
        {
            return (GetRecordingsResponse)base.Execute();
        }
        
        /// <summary>
        /// Required	The meeting ID that identifies the meeting you are attempting to join.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("The meeting ID that identifies the meeting you are attempting to join.")]
        public string MeetingID
        {
            get { return GetParamByName("meetingID"); }
            set { AddParameters("meetingID", value); }
        }
    }

    public class GetRecordingsResponse : BaseResponse
    {
        private Recordings _recordings = new Recordings();
        public Recordings Recordings
        {
            get { return _recordings; }
        }

        protected override void ProcessChildNodes(XmlNode node, Hashtable hashtable)
        {
            if (node.Name.ToLower() == "recordings")
            {
                _recordings.Clear();
                Recording _recording = new Recording();
                foreach (XmlNode recordings in node.ChildNodes)
                {
                    foreach (XmlNode recording in recordings.ChildNodes)
                    {
                        _recording.AddHashTable(recording.Name, recording.InnerText);
                    }
                    _recording.Response = node.InnerXml;
                    _recordings.Add(_recording);
                }
            }
            //base.ProcessChildNodes(node, hashtable);
        }
    }

    public class Recordings : List<Recording>
    { }

    public class Recording : BaseResponse
    {
        public string RecordID
        {
            get { return GetValueByName("recordID"); }
            set { this.AddHashTable("recordID", value); }
        }

        public string MeetingID
        {
            get { return GetValueByName("meetingID"); }
            set { this.AddHashTable("meetingID", value); }
        }

        public string Name
        {
            get { return GetValueByName("name"); }
            set { this.AddHashTable("name", value); }
        }

        public bool Published
        {
            get
            {
                string published = GetValueByName("published");
                return published.ToLower() == "true";
            }
            set { this.AddHashTable("published", value.ToString()); }
        }

        public DateTime StartTime
        {
            get
            {
                string startTime = GetValueByName("startTime");
                return General.MillisecondsToDateTime(startTime);
            }
            set
            {
                string mills = General.DateTimeToMilliseconds(value);
                this.AddHashTable("startTime", mills);
            }
        }

        public DateTime EndTime
        {
            get
            {
                string endTime = GetValueByName("endTime");
                return General.MillisecondsToDateTime(endTime);
            }
            set
            {
                string mills = General.DateTimeToMilliseconds(value);
                this.AddHashTable("endTime", mills);
            }
        }
    }

    public class Format : BaseResponse
    {
        public string Type
        {
            get { return GetValueByName("type"); }
            set { this.AddHashTable("type", value); }
        }

        public string Url
        {
            get { return GetValueByName("url"); }
            set { this.AddHashTable("url", value); }
        }

        public string Length
        {
            get { return GetValueByName("length"); }
            set { this.AddHashTable("length", value); }
        }
    }

    public class Metadata : BaseResponse
    {
        public string Title
        {
            get { return GetValueByName("title"); }
            set { this.AddHashTable("title", value); }
        }

        public string Subject
        {
            get { return GetValueByName("subject"); }
            set { this.AddHashTable("subject", value); }
        }

        public string Description
        {
            get { return GetValueByName("description"); }
            set { this.AddHashTable("description", value); }
        }

        public string Creator
        {
            get { return GetValueByName("creator"); }
            set { this.AddHashTable("creator", value); }
        }

        public string Contributor
        {
            get { return GetValueByName("contributor"); }
            set { this.AddHashTable("contributor", value); }
        }

        public string Language
        {
            get { return GetValueByName("language"); }
            set { this.AddHashTable("language", value); }
        }
    }

    public class GetRecordings
    {
        private GetRecordingsRequest _request;
        [CategoryAttribute("Get Meeting Info Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Get Meeting Info request")]
        public GetRecordingsRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private GetRecordingsResponse _response;
        [CategoryAttribute("Get Meeting Info Response")]
        [DisplayName("Get Meeting Info response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public GetRecordingsResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
