﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class IsMeetingRunningResponse : BaseResponse
    {
        public bool IsRunning
        {
            get
            {
                string running = GetValueByName("running").ToLower();
                return running == "true";
            }
        }
    }

    public class IsMeetingRunningRequest : BaseRequest
    {
        public IsMeetingRunningRequest()
            : base()
        {
            this._methodName = "isMeetingRunning";
            this._response = new IsMeetingRunningResponse();
        }

        public new IsMeetingRunningResponse Execute()
        {
            return (IsMeetingRunningResponse)base.Execute();
        }

        /// <summary>
        /// Required	A meeting ID that can be used to identify this meeting by the 3rd-party application.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("A meeting ID that can be used to identify this meeting by the 3rd-party application.")]
        public string MeetingID
        {
            get { return GetParamByName("meetingID"); }
            set { AddParameters("meetingID", value); }
        }
    }

    public class IsMeetingRunning
    {
        private IsMeetingRunningRequest _request;
        [CategoryAttribute("Is Meeting Running Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Is Meeting Running request")]
        public IsMeetingRunningRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private IsMeetingRunningResponse _response;
        [CategoryAttribute("Is Meeting Running Response")]
        [DisplayName("Is Meeting Running response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public IsMeetingRunningResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
