﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class JoinMeetingRequest : BaseRequest
    {
        public JoinMeetingRequest()
        {
            this._methodName = "join";
            this._response = new JoinMeetingResponse(); 
        }

        public new JoinMeetingResponse Execute()
        {
            return (JoinMeetingResponse)base.Execute();
        }

        /// <summary>
        /// Required	The full name that is to be used to identify this user to other conference attendees.
        /// </summary>
        [DisplayName("Full Name (required)")]
        [Description("The full name that is to be used to identify this user to other conference attendees.")]
        public string FullName
        {
            get 
            { 
                string encoded = GetParamByName("fullName");
                return WebUtility.UrlDecode(encoded);
            }
            set 
            {
                string encoded = WebUtility.UrlEncode(value);
                AddParameters("fullName", encoded); 
            }
        }

        /// <summary>
        /// Required	The meeting ID that identifies the meeting you are attempting to join.
        /// </summary>
        [DisplayName("Meeting ID (required)")]
        [Description("The meeting ID that identifies the meeting you are attempting to join.")]
        public string MeetingID
        {
            get { return GetParamByName("meetingID"); }
            set { AddParameters("meetingID", value); }
        }

        /// <summary>
        /// Required	The password that this attendee is using. If the moderator password is supplied, he will be given moderator status (and the same for attendee password, etc)
        /// </summary>
        [DisplayName("Password (required)")]
        [Description("The password that this attendee is using. If the moderator password is supplied, he will be given moderator status (and the same for attendee password, etc)")]
        public string Password
        {
            get { return GetParamByName("password"); }
            set { AddParameters("password", value); }
        }

        /// <summary>
        /// Optional	Third-party apps using the API can now pass createTime parameter (which was created in the create call), BigBlueButton will ensure it matches the ‘createTime’ for the session. If they differ, BigBlueButton will not proceed with the join request. This prevents a user from reusing their join URL for a subsequent session with the same meetingID.
        /// </summary>
        [DisplayName("Create Time (optional)")]
        [Description("Third-party apps using the API can now pass createTime parameter (which was created in the create call), BigBlueButton will ensure it matches the ‘createTime’ for the session. If they differ, BigBlueButton will not proceed with the join request. This prevents a user from reusing their join URL for a subsequent session with the same meetingID.")]
        public DateTime CreateTime
        {
            get 
            {
                string createTime = GetParamByName("createTime");
                return General.MillisecondsToDateTime(createTime);
            }
            set 
            {
                string mills = General.DateTimeToMilliseconds(value);
                AddParameters("createTime", mills); 
            }
        }

        /// <summary>
        /// Optional	An identifier for this user that will help your application to identify which person this is. This user ID will be returned for this user in the getMeetingInfo API call so that you can check
        /// </summary>
        [DisplayName("User ID (optional)")]
        [Description("An identifier for this user that will help your application to identify which person this is. This user ID will be returned for this user in the getMeetingInfo API call so that you can check")]
        public string UserID
        {
            get { return GetParamByName("userID"); }
            set { AddParameters("userID", value); }
        }

        /// <summary>
        /// Optional	If you want to pass in a custom voice-extension when a user joins the voice conference using voip. This is useful if you want to collect more info in you Call Detail Records about the user joining the conference. You need to modify your /etc/asterisk/bbb-extensions.conf to handle this new extensions.
        /// </summary>
        [DisplayName("WebVoiceConf (optional)")]
        [Description("If you want to pass in a custom voice-extension when a user joins the voice conference using voip. This is useful if you want to collect more info in you Call Detail Records about the user joining the conference. You need to modify your /etc/asterisk/bbb-extensions.conf to handle this new extensions.")]
        public string WebVoiceConf
        {
            get { return GetParamByName("webVoiceConf"); }
            set { AddParameters("webVoiceConf", value); }
        }

        /// <summary>
        /// Optional	The token returned by a setConfigXML API call. This causes the BigBlueButton client to load the config.xml associated with the token (not the default config.xml)
        /// </summary>
        [DisplayName("Config Token (optional)")]
        [Description("The token returned by a setConfigXML API call. This causes the BigBlueButton client to load the config.xml associated with the token (not the default config.xml)")]
        public string ConfigToken
        {
            get { return GetParamByName("configToken"); }
            set { AddParameters("configToken", value); }
        }

        /// <summary>
        /// Optional	The link for the user's avatar to be displayed when displayAvatar in config.xml is set to true.
        /// </summary>
        [DisplayName("Avatar URL (optional)")]
        [Description("The link for the user's avatar to be displayed when displayAvatar in config.xml is set to true.")]
        public string AvatarURL
        {
            get { return GetParamByName("avatarURL"); }
            set { AddParameters("avatarURL", value); }
        }

        /// <summary>
        /// Optional (Experimental)	The default behaviour of the JOIN API is to redirect the browser to the Flash client when the JOIN call succeeds. There have been requests if it's possible to embed the Flash client in a "container" page and that the client starts as a hidden DIV tag which becomes visible on the successful JOIN. Setting this variable to FALSE will not redirect the browser but returns an XML instead whether the JOIN call has succeeded or not. The third party app is responsible for displaying the client to the user.
        /// </summary>
        [DisplayName("Redirect (optional)")]
        [Description("The default behaviour of the JOIN API is to redirect the browser to the Flash client when the JOIN call succeeds. There have been requests if it's possible to embed the Flash client in a \"container\" page and that the client starts as a hidden DIV tag which becomes visible on the successful JOIN. Setting this variable to FALSE will not redirect the browser but returns an XML instead whether the JOIN call has succeeded or not. The third party app is responsible for displaying the client to the user.")]
        public string Redirect
        {
            get { return GetParamByName("redirect"); }
            set { AddParameters("redirect", value); }
        }

        /// <summary>
        /// Optional (Experimental)	Some third party apps what to display their own custom client. These apps can pass the URL containing the custom client and when redirect is not set to false, the browser will get redirected to the value of clientURL
        /// </summary>
        [DisplayName("Client URL (optional)")]
        [Description("String	Some third party apps what to display their own custom client. These apps can pass the URL containing the custom client and when redirect is not set to false, the browser will get redirected to the value of clientURL")]
        public string ClientURL
        {
            get { return GetParamByName("clientURL"); }
            set { AddParameters("clientURL", value); }
        }
    }

    public class JoinMeetingResponse : BaseResponse
    {
        public bool IsRunning
        {
            get
            {
                string running = GetValueByName("running");
                return running == "true";
            }
        }
    }

    public class JoinMeeting
    {
        private JoinMeetingRequest _request;
        [CategoryAttribute("Join Meeting Request")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        [DisplayName("Join meeting request")]
        public JoinMeetingRequest Request
        {
            get { return _request; }
            set { _request = value; }
        }

        private JoinMeetingResponse _response;
        [CategoryAttribute("Join Meeting Response")]
        [DisplayName("Join meeting response")]
        [TypeConverter(typeof(ExpandableObjectConverter))]
        public JoinMeetingResponse Response
        {
            get { return _response; }
            set { _response = value; }
        }
    }
}
