﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml;
using System.Xml.Linq;
using System.Collections;

/// <summary>
/// This project is developed by Andrey A. Fedorov from BizKit.ru.
/// E-mail me at fedorov@bizkit.ru for any support on this code or even support on BigBlueButton.
/// </summary>
namespace BigBlueButtonSDK
{
    public class BaseResponse
    {
        public string ReturnCode
        {
            get 
            {
                return GetValueByName("returncode");
            }
        }

        public bool IsSuccess
        {
            get
            {
                return this.ReturnCode.ToLower() == "success";
            }
        }

        public string Message
        {
            get 
            {
                return GetValueByName("message");
            }
        }

        public string MessageKey
        {
            get 
            {
                return GetValueByName("messagekey");
            }
        }

        private Hashtable _hashTable = new Hashtable(StringComparer.InvariantCultureIgnoreCase);
        public Hashtable HashTable
        {
            get 
            { 
                return _hashTable; 
            }
        }

        protected string GetValueByName(string Name)
        {
            Name = Name.Trim();
            if (_hashTable.ContainsKey(Name))
            {
                return (string)_hashTable[Name];
            }
            return string.Empty;
        }

        protected string _response = string.Empty;
        public string Response
        {
            get { return _response; }
            set
            {
                this.FillHashTable(value, "response");//, ref _hashTable);
                _response = value;
            }
        }

        public void AddHashTable(string key, string value) //, ref Hashtable hashTable)
        {
            key = key.Trim();
            if (_hashTable.ContainsKey(key))
                _hashTable[key] = value;
            else
                _hashTable.Add(key, value);
        }

        protected virtual void ProcessChildNodes(XmlNode node, Hashtable hashtable)
        {

        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="value">XML response</param>
        /// <param name="RootNodeName"></param>
        /// <param name="hashTable"></param>
        protected void FillHashTable(string value, string RootNodeName) //, ref Hashtable hashTable)
        {
            if (!string.IsNullOrEmpty(value))
            {
                if (value.ToLower().Contains("<response") && value.ToLower().Contains("<returncode"))
                {
                    XmlDocument xmlResponse = new XmlDocument();
                    xmlResponse.LoadXml(value);

                    if (xmlResponse != null)
                    {
                        XmlNodeList response = xmlResponse.GetElementsByTagName(RootNodeName);
                        if (response != null && response.Count > 0)
                        {
                            foreach (XmlNode node in response[0].ChildNodes)
                            {
                                if (node.HasChildNodes && node.ChildNodes[0].HasChildNodes)
                                    this.ProcessChildNodes(node, _hashTable);
                                //this.AddHashTable(node.Name, node.InnerText, ref hashTable);
                                else
                                    if (!string.IsNullOrEmpty(node.InnerText))
                                        this.AddHashTable(node.Name, node.InnerText); //, ref hashTable);
                            }
                        }
                    }
                }
            }
        }

        public BaseResponse(string ResponseXML)
        {
            this.Response = ResponseXML;
        }

        public BaseResponse()
        {

        }
    }




}
